@@ -34,10 +34,8 @@ rbg_rval2cstr(VALUE *str)
 {
     StringValue(*str);
 
-#ifdef HAVE_RUBY_ENCODING_H
     if (rb_enc_get(*str) != rb_utf8_encoding())
         *str = rb_str_export_to_enc(*str, rb_utf8_encoding());
-#endif
 
     return StringValueCStr(*str);
 }
@@ -142,11 +140,7 @@ rbg_cstr2rval_len(const gchar *str, gsize len)
     if (str == NULL)
         return Qnil;
 
-#ifdef HAVE_RUBY_ENCODING_H
     return rb_external_str_new_with_enc(str, len, rb_utf8_encoding());
-#else
-    return rb_str_new(str, len);
-#endif
 }
 
 struct rbg_cstr2rval_len_free_args {
@@ -185,7 +179,6 @@ rbg_cstr2rval_with_encoding(const gchar *str, const gchar *encoding)
     return str != NULL ? CSTR2RVAL_LEN_ENC(str, strlen(str), encoding) : Qnil;
 }
 
-#ifdef HAVE_RUBY_ENCODING_H
 VALUE
 rbg_cstr2rval_len_with_encoding(const gchar *str, gsize len,
                                 const gchar *encoding)
@@ -198,17 +191,6 @@ rbg_cstr2rval_len_with_encoding(const gchar *str, gsize len,
                                             rb_enc_find(encoding) :
                                             rb_utf8_encoding());
 }
-#else
-VALUE
-rbg_cstr2rval_len_with_encoding(const gchar *str, gsize len,
-                                G_GNUC_UNUSED const gchar *encoding)
-{
-    if (str == NULL)
-        return Qnil;
-
-    return rb_str_new(str, len);
-}
-#endif
 
 static VALUE
 rbg_cstr2rval_free_body(VALUE str)
@@ -238,11 +220,8 @@ rbg_cstr2rval_with_free(gchar *str)
     return rbg_cstr2rval_free(str);
 }
 
-#ifdef HAVE_RUBY_ENCODING_H
 static rb_encoding *filename_encoding_if_not_utf8;
-#endif
 
-#ifdef HAVE_RUBY_ENCODING_H
 static VALUE
 rbg_filename_to_ruby_body(VALUE filename)
 {
@@ -267,12 +246,10 @@ rbg_filename_to_ruby_ensure(VALUE filename)
 
     return Qnil;
 }
-#endif
 
 VALUE
 rbg_filename_to_ruby(const gchar *filename)
 {
-#ifdef HAVE_RUBY_ENCODING_H
     gchar *filename_utf8;
     gsize written;
     GError *error = NULL;
@@ -289,15 +266,11 @@ rbg_filename_to_ruby(const gchar *filename)
 
     return rb_ensure(rbg_filename_to_ruby_body, (VALUE)filename_utf8,
                      rbg_filename_to_ruby_ensure, (VALUE)filename_utf8);
-#else
-    return CSTR2RVAL(filename);
-#endif
 }
 
 VALUE
 rbg_filename_to_ruby_free(gchar *filename)
 {
-#ifdef HAVE_RUBY_ENCODING_H
     gchar *filename_utf8;
     gsize written;
 
@@ -318,15 +291,11 @@ rbg_filename_to_ruby_free(gchar *filename)
 
     return rb_ensure(rbg_filename_to_ruby_body, (VALUE)filename_utf8,
                      rbg_filename_to_ruby_ensure, (VALUE)filename_utf8);
-#else
-    return CSTR2RVAL_FREE(filename);
-#endif
 }
 
 gchar *
 rbg_filename_from_ruby(VALUE filename)
 {
-#ifdef HAVE_RUBY_ENCODING_H
     gchar *retval;
     gsize written;
     GError *error = NULL;
@@ -345,9 +314,6 @@ rbg_filename_from_ruby(VALUE filename)
         RAISE_GERROR(error);
 
     return retval;
-#else
-    return g_strdup(RVAL2CSTR(filename));
-#endif
 }
 
 struct rval2strv_args {
@@ -888,47 +854,79 @@ rbg_check_hash_type (VALUE object)
 void
 rbg_scan_options (VALUE options, ...)
 {
-    VALUE original_options = options;
-    VALUE available_keys;
     const char *key;
-    VALUE *value;
+    gsize n_keys = 0;
+    gsize n_found_keys = 0;
     va_list args;
+    va_list args_copy;
 
-    if (NIL_P(options)) {
-        options = rb_hash_new();
-    } else {
+    if (!NIL_P(options)) {
+        VALUE original_options = options;
         options = rbg_check_hash_type(options);
-        if (options == original_options) {
-            options = rb_funcall(options, rb_intern("dup"), 0);
-        } else if (NIL_P(options)) {
+        if (NIL_P(options)) {
             rb_raise(rb_eArgError,
                      "options must be Hash or nil: %+" PRIsVALUE,
                      original_options);
         }
     }
 
-    available_keys = rb_ary_new();
     va_start(args, options);
+    va_copy(args_copy, args);
     key = va_arg(args, const char *);
+    n_keys = 0;
+    n_found_keys = 0;
     while (key) {
-        VALUE rb_key;
-        value = va_arg(args, VALUE *);
+        VALUE *value;
 
-        rb_key = ID2SYM(rb_intern(key));
-        rb_ary_push(available_keys, rb_key);
-        *value = rb_funcall(options, rb_intern("delete"), 1, rb_key);
+        value = va_arg(args, VALUE *);
+        if (NIL_P(options)) {
+            *value = Qnil;
+        } else {
+            VALUE rb_key;
+            rb_key = ID2SYM(rb_intern(key));
+            if (RTEST(rb_funcall(options, rb_intern("key?"), 1, rb_key))) {
+                n_found_keys++;
+            }
+            *value = rb_hash_aref(options, rb_key);
+        }
+        n_keys++;
 
         key = va_arg(args, const char *);
     }
     va_end(args);
 
-    if (RVAL2CBOOL(rb_funcall(options, rb_intern("empty?"), 0)))
+    if (NIL_P(options)) {
         return;
+    }
 
-    rb_raise(rb_eArgError,
-             "unexpected key(s) exist: %s: available keys: %s",
-             rbg_inspect(rb_funcall(options, rb_intern("keys"), 0)),
-             rbg_inspect(available_keys));
+    if (n_found_keys == RHASH_SIZE(options)) {
+        return;
+    }
+
+    {
+        VALUE rb_available_keys;
+
+        rb_available_keys = rb_ary_new();
+        key = va_arg(args_copy, const char *);
+        while (key) {
+            VALUE rb_key;
+
+            va_arg(args_copy, VALUE *);
+            rb_key = ID2SYM(rb_intern(key));
+            rb_ary_push(rb_available_keys, rb_key);
+            key = va_arg(args_copy, const char *);
+        }
+        va_end(args_copy);
+
+        rb_raise(rb_eArgError,
+                 "unexpected key(s) exist: %+" PRIsVALUE
+                 ": available keys: %+" PRIsVALUE,
+                 rb_funcall(rb_funcall(options, rb_intern("keys"), 0),
+                            rb_intern("-"),
+                            1,
+                            rb_available_keys),
+                 rb_available_keys);
+    }
 }
 
 #if 0
@@ -1013,9 +1011,7 @@ extern void Init_glib2(void);
 void
 Init_glib2(void)
 {
-#ifdef HAVE_RUBY_ENCODING_H
     const gchar **filename_charsets;
-#endif
 
     id_inspect = rb_intern("inspect");
 
@@ -1096,7 +1092,6 @@ Init_glib2(void)
     rb_define_const(RG_TARGET_NAMESPACE, "SEARCHPATH_SEPARATOR", CSTR2RVAL(G_SEARCHPATH_SEPARATOR_S));
 
     /* discover and store glib filename encoding */
-#ifdef HAVE_RUBY_ENCODING_H
     if (g_get_filename_charsets(&filename_charsets)
         || filename_charsets == NULL
         || filename_charsets[0] == NULL
@@ -1108,7 +1103,6 @@ Init_glib2(void)
     } else {
         filename_encoding_if_not_utf8 = rb_enc_find(filename_charsets[0]);
     }
-#endif
 
 /* Don't implement them.
 #define     G_DIR_SEPARATOR_S
@@ -1193,4 +1187,7 @@ union       GDoubleIEEE754;
     Init_glib_variant();
     Init_glib_regex();
     Init_glib_matchinfo();
+    Init_glib_date_time();
+    Init_glib_time_zone();
+    Init_glib_bytes();
 }
